
// Mainwindow

#include <pybind11/include/pybind11/pybind11.h>
#include <pybind11/include/pybind11/embed.h>

#include <QSurfaceFormat>


#include "OpenFlipper/Core/Core.hh"
#include "common/glew_wrappers.hh"
#include "OpenFlipper/BasePlugin/PluginFunctionsCore.hh"

// Qt
#include <qgl.h>

// stdc++
#include <csignal>
#include <regex>

#include <QCommandLineParser>

#if ( defined(WIN32))
  #define NO_EXECINFO
#endif

#ifndef NO_EXECINFO
#include <execinfo.h>
#endif

#ifdef USE_OPENMP
#endif

#ifdef _WIN32
#  include <windows.h>
#  include <errhandlingapi.h>
#endif


#include <OpenGL/OpenGLDetection.hh>

#include <PythonInterpreter/PythonInterpreter.hh>

int add(int i, int j) {
    return i + j;
}


void startup() {
#ifdef _WIN32
    // This make crashes visible - without them, starting the
    // application from cmd.exe or powershell can surprisingly hide
    // any signs of a an application crash!
    SetErrorMode(0); // 0: Use the system default, which is to display all error dialog boxes.
#endif

    // Interpreter is already running, so we don't need this!
    //pybind11::initialize_interpreter();


    char  arg0[] = "OpenFlipper";
    char  arg1[] = "-c";
    char  arg2[] = "-d";
    char* argv[] = { &arg0[0], &arg1[0], &arg2[0] , NULL };
    int   argc   = (int)(sizeof(argv) / sizeof(argv[0])) - 1;

    OpenFlipper::Options::argc(&argc);
    //OpenFlipper::Options::argv(&argv);


    pybind11::object module = pybind11::module::import("libopenflipperpython");
    QString modulepath = QString(module.attr("__file__").cast<std::string>().c_str());
    QString moduleDirectory = modulepath.section("/",0,-2);

    OpenFlipper::Options::setApplicationDir(moduleDirectory);

    // Set organization and application names
    QCoreApplication::setOrganizationName("VCI");
    QCoreApplication::setApplicationName(TOSTRING(PRODUCT_STRING));
    QCoreApplication::setApplicationVersion(OpenFlipper::Options::coreVersion());

    // initialize a core application to check for initialization Paths
    QCoreApplication* coreApp = new QCoreApplication(argc, argv);
    OpenFlipper::Options::initializeSettings();

    // only one application is allowed so delete the core application
    // once cmdline parsing is done
    delete coreApp;

    OpenFlipper::Options::windowTitle(TOSTRING(PRODUCT_STRING)" v" + OpenFlipper::Options::coreVersion());


    if ( !OpenFlipper::Options::nogui() ) {

        // OpenGL check
        QApplication::setAttribute(Qt::AA_ShareOpenGLContexts);

        // Try creating a valid OpenGL context
        /******************************/

            // Get a valid context format
            QSurfaceFormat resultFormat = getContextFormat();

            // Set temporary(!) OpenGL settings
            OpenFlipper::Options::samples(resultFormat.samples(), true);
            OpenFlipper::Options::glStereo(resultFormat.stereo(), true);
            OpenFlipper::Options::glVersion(resultFormat.version(), true);
            OpenFlipper::Options::coreProfile(resultFormat.profile() == QSurfaceFormat::CoreProfile, true);

            // Create the actual context
            QSurfaceFormat::setDefaultFormat(resultFormat);
            QApplication app(argc, argv);
            QOffscreenSurface *surface = new QOffscreenSurface();
            surface->create();

            // Make the globally shared OpenGLContext current
            QOpenGLContext::globalShareContext()->makeCurrent(surface);
            /******************************/


            // Check whether there is OpenGL support. If not, return.
            if ( !QGLFormat::hasOpenGL() ) {
                std::cerr << "This system has no OpenGL support.\n";
                return ;
            }
 std::cerr << "ssss" << std::endl;
            // create core ( this also reads the ini files )
            Core * w = new Core( );
#ifdef PYTHON_ENABLED
            //setCorePointer(w);
#endif
 std::cerr << "AAddddA" << std::endl;
            QString tLang = OpenFlipperSettings().value("Core/Language/Translation","en_US").toString();

            if (tLang == "locale")
                tLang = QLocale::system().name();

            // Install translator for qt internals
            QTranslator qtTranslator;
            qtTranslator.load("qt_" + tLang, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
            app.installTranslator(&qtTranslator);

            // install translator for Core Application
            QString translationDir = OpenFlipper::Options::translationsDirStr() + QDir::separator();
            QDir dir(translationDir);
            dir.setFilter(QDir::Files);

            QFileInfoList list = dir.entryInfoList();

            for (int i = 0; i < list.size(); ++i) {
                QFileInfo fileInfo = list.at(i);

                if ( fileInfo.baseName().contains(tLang) ){
                    QTranslator* myAppTranslator = new QTranslator();

                    if ( myAppTranslator->load( fileInfo.filePath() ) )
                    {
                        app.installTranslator(myAppTranslator);
                    } else
                    {
                        delete myAppTranslator;
                    }
                }
            }

#ifndef __APPLE__
            initGlew();
#endif

            // After setting all Options from command line, build the real gui
            w->init();

            app.exec();

        }
    pybind11::finalize_interpreter();
    }

PYBIND11_MODULE(libopenflipperpython , m) {

  m.doc() = "OpenFlipper test module";

  m.def("startup", &startup, "Start the openflipper core");
  m.def("add", &add, "A function which adds two numbers");
}


