import os, shutil, subprocess, sys

print("-----------------------")
print("Running tests via run_tests.py for "+sys.platform)

ctest_executable = ""

# setup operating system
if sys.platform == "linux" or sys.platform == "linux2":
    ctest_executable = "ctest"
elif sys.platform == "darwin":
    ctest_executable = os.path.join("/","opt", "local", "bin", "ctest")
elif sys.platform == "win32":
    ctest_executable = os.path.join("C:","Program Files","CMake","bin","ctest")
    cmake_executable = os.path.join("C:","Program Files","CMake","bin","cmake")

    print("===============================================")
    print("Python test runner script settings")
    print("===============================================")
    print("Building with:")

    print("ARCHITECTURE        : " + os.environ.get('ARCHITECTURE'))
    print("BUILD_PLATFORM      : " + os.environ.get('BUILD_PLATFORM'))
    print("GTESTVERSION        : " + os.environ.get('GTESTVERSION'))
    print("GENERATOR           : " + os.environ.get('GENERATOR'))
    print("VS_PATH             : " + os.environ.get('VS_PATH'))
    if "LIBPATH" in os.environ:
      print("LIBPATH             : " + os.environ.get('LIBPATH'))
    else:
      print("LIBPATH             : UNSET" )

    if "LIBPATH_BASE" in os.environ:
      print("LIBPATH_BASE        : " + os.environ.get('LIBPATH_BASE'))
    else:
      print("LIBPATH_BASE        : UNSET")
  
    if "QT_INSTALL_PATH" in os.environ:
      print("QT_INSTALL_PATH     : " + os.environ.get('QT_INSTALL_PATH'))
    else:
      print("QT_INSTALL_PATH     : UNSET")

    print("CMAKE_CONFIGURATION :" + os.environ.get('CMAKE_CONFIGURATION'))
    print("===============================================")
    print("===============================================")
    print("")
    print("===============================================")
    print("")
    print("")
    print("Running Build environment checks")
    print("Current working directory: " + os.getcwd( ))

    
    if os.path.exists(os.environ.get('LIBPATH_BASE')):
      print("LIBPATH_BASE " + os.environ.get('LIBPATH_BASE') + " is ok")
    else:
      print("ERROR: LIBPATH_BASE " + os.environ.get('LIBPATH_BASE') + " not found!")
      exit(10)

    if "QT_INSTALL_PATH" in os.environ:
       if os.path.exists(os.environ.get('QT_INSTALL_PATH')):
         print("QT_INSTALL_PATH " + os.environ.get('QT_INSTALL_PATH') + " is ok")
       else:
         print("ERROR: QT_INSTALL_PATH " + os.environ.get('QT_INSTALL_PATH') + " not found!")
         exit(10)

    print("Entering rel directory")
    os.chdir(os.path.join(os.path.abspath(os.path.curdir),u'rel')) 
    print("Current working directory: " + os.getcwd( ))

    if os.path.exists("CMakeCache.txt"):
        print("Remove old CMakeCache.txt")
        os.remove("CMakeCache.txt")

    if os.path.exists("CTestTestfile.cmake"):
        print("Remove old CTestTestfile.cmake")
        os.remove("CTestTestfile.cmake")

    if os.path.exists("DartConfiguration.tcl"):
        print("Remove old DartConfiguration.tcl")
        os.remove("DartConfiguration.tcl")

    print("", flush=True)

    gtest_prefix        = "-DGTEST_PREFIX=\"" + os.environ.get('LIBPATH_BASE')+"\\" + os.environ.get('ARCHITECTURE') + "\\" +os.environ.get('GTESTVERSION') + "\""
    cmake_generator     = os.environ.get('GENERATOR')
    cmake_configuration_list = os.environ.get('CMAKE_CONFIGURATION').split()

    print(cmake_configuration_list)

    cmake_parameters = [cmake_executable, gtest_prefix , "-G", cmake_generator ,"-DCMAKE_BUILD_TYPE=Release","-DOPENFLIPPER_BUILD_UNIT_TESTS=TRUE" ]

    cmake_parameters.extend(cmake_configuration_list)
    cmake_parameters.append("..")

    print(cmake_parameters)
 
    cmake_call = subprocess.run( cmake_parameters )

    print("Cmake call info: ")
    print(cmake_call)
    print("Return Code: ", cmake_call.returncode)

    if cmake_call.returncode != 0:
       print("CMake error! exiting")
       exit(cmake_call.returncode)

    print("Entering tests directory")
    os.chdir(os.path.join(os.path.abspath(os.path.curdir),u'tests')) 
    print("Current working directory: " + os.getcwd( ))

    # preparing DLLs to make sure we can run the application
    if not os.path.exists("../Build"):
       message("Error: the folder ../Build has not been found!")
       exit(1)

    src_files = os.listdir("../Build")
    for file_name in src_files:
        full_file_name = os.path.join("../Build", file_name)
        if os.path.isfile(full_file_name) and full_file_name.endswith(".dll"):
           print("Copyining" + full_file_name + " to testBinaries")
           shutil.copy(full_file_name, "testBinaries")




os.chdir(os.path.join(os.getcwd(), os.path.pardir))  # cd ..
print("Working directory for test cleanup: " + os.getcwd())

try:
    shutil.rmtree(os.path.join(os.getcwd(), "tests", "testResults")) # delete test results if exists
except FileNotFoundError :
    print("testResults not found")

try:
    os.remove(os.path.join(os.getcwd(), "CTestResults.xml")) # delete test results if exists
except FileNotFoundError:
    print("CTestResults not found")

print("Creating dir for test results")
os.mkdir(os.path.join(os.getcwd(), "tests", "testResults"))

print("Working directory for test execution: " + os.getcwd(), flush=True)

# call ctest
ctest = subprocess.run(
    [ctest_executable, "-C", "Release", "--no-tests=error", "--no-compress-output", "--output-on-failure"])

print("Return Code: ", ctest.returncode)

print("-----------------------")

sys.exit(ctest.returncode)
